# 10 px == 1 metre
phy_mess: float = 10.0 / 1
phy_velocity: float = (((1000.0 / 60.0) / 60.0) * phy_mess)

# Mouse buttons
MB_LEFT = 1
MB_MIDDLE = 2
MB_RIGHT = 3
MB_WHEEL = MB_MIDDLE
MB_WHEEL_UP = 4
MB_WHEEL_DOWN = 5

# Virtual keys
VK_LEFT = 1073741904
VK_RIGHT = 1073741903
VK_UP = 1073741906
VK_DOWN = 1073741905

VK_ESCAPE = 27
VK_BACKSPACE = 8
VK_ENTER = 13
VK_TAB = 9
VK_SPACE = 32

VK_CAPS_LOCK = 1073741881
VK_SCROLL_LOCK = 1073741895
VK_PAUSE = 1073741896

VK_AT = 64  # @
VK_PERCENT = 37  # %
VK_AMPERSAND = 38  # &
VK_TILDE = 126  # ~
VK_BAR = 124  # |
VK_UNDERSCORE = 95  # _
VK_CARET = 94  # ^
VK_DOLLAR = 36  # $
VK_HASH = 35  #
VK_BACKSLASH = 92  # \
VK_SHARP = VK_HASH
VK_UNDERBAR = VK_UNDERSCORE

VK_BANG = 33  # !
VK_QUESTION = 63  # ?
VK_ACCENT = 96  # `
VK_PERIOD = 46  # .
VK_COMMA = 44  # ,
VK_QUOTE = 96  # "
VK_APOSTROPHE = 39  # '

VK_HYPHEN = 45  # -
VK_COLON = 58  # :
VK_SEMICOLON = 59  # ;

VK_PARENTHESES_L = 40  # (
VK_PARENTHESES_R = 41  # )
VK_BRACE_L = 123  # {
VK_BRACE_R = 125  # }
VK_BRACKET = 91  # [
VK_BRACKET = 93  # ]
VK_BRACKET_ANGLE_L = 60  # <
VK_BRACKET_ANGLE_R = 62  # >

VK_EQUAL = 61  # =
VK_PLUS = 43  # +
VK_MINUS = VK_HYPHEN
VK_ASTERISK = 42  # *
VK_SLASH = 47  # /

VK_LESS = VK_BRACKET_ANGLE_L
VK_GREAT = VK_BRACKET_ANGLE_R
VK_MULTIPLY = VK_ASTERISK
VK_DIVIDE = VK_SLASH
VK_XOR = VK_CARET

VK_INSERT = 1073741897
VK_DELETE = 127
VK_HOME = 1073741898
VK_END = 1073741901
VK_PAGE_UP = 1073741899
VK_PAGE_DOWN = 1073741902

VK_A = 65
VK_B = 66
VK_C = 67
VK_D = 68
VK_E = 69
VK_F = 70
VK_G = 71
VK_H = 72
VK_I = 73
VK_J = 74
VK_K = 75
VK_L = 76
VK_M = 77
VK_N = 78
VK_O = 79
VK_P = 80
VK_Q = 81
VK_R = 82
VK_S = 83
VK_T = 84
VK_U = 85
VK_V = 86
VK_W = 87
VK_X = 88
VK_Y = 89
VK_Z = 90

VK_SA = 97
VK_SB = 98
VK_SC = 99
VK_SD = 100
VK_SE = 101
VK_SF = 102
VK_SG = 103
VK_SH = 104
VK_SI = 105
VK_SJ = 106
VK_SK = 107
VK_SL = 108
VK_SM = 109
VK_SN = 110
VK_SO = 111
VK_SP = 112
VK_SQ = 113
VK_SR = 114
VK_SS = 115
VK_ST = 116
VK_SU = 117
VK_SV = 118
VK_SW = 119
VK_SX = 120
VK_SY = 121
VK_SZ = 122

VK_F01 = 1073741882
VK_F02 = 1073741883
VK_F03 = 1073741884
VK_F04 = 1073741885
VK_F05 = 1073741886
VK_F06 = 1073741887
VK_F07 = 1073741888
VK_F08 = 1073741889
VK_F09 = 1073741890
VK_F10 = 1073741891
VK_F11 = 1073741892
VK_F12 = 1073741893


class INPUT_STATES:
    NONE = 0
    PRESSED = 1
    ING = 2
    RELEASED = 3


class MASKS:
    RECTANGLE = 0
    PRECISE = 1
    CIRCLE = 2
